/*
 *
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * 5724-U18
 *
 * (C) COPYRIGHT IBM CORP. 2006,2014
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 *
 */
package psdi.iface.mos;

import java.io.UnsupportedEncodingException;
import java.rmi.RemoteException;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.jdom.Document;

import com.ibm.tivoli.maximo.log.LogInfo;
import com.ibm.tivoli.maximo.log.MaxLoggerContext;
import com.ibm.tivoli.maximo.log.ThreadLoggerCache;

import psdi.iface.bidi.maximo.MaximoBidiTransformer;
import psdi.iface.mic.MicService;
import psdi.iface.mic.MicUtil;
import psdi.iface.util.SecurityUtil;
import psdi.iface.util.XMLUtils;
import psdi.iface.webservices.WSRegistryCache;
import psdi.iface.webservices.WSRegistryInfo;
import psdi.security.UserInfo;
import psdi.server.MXServer;
import psdi.server.MaxPropCache;
import psdi.util.MXApplicationException;
import psdi.util.MXException;
import psdi.util.MXFormat;
import psdi.util.MXSession;

/**
 * EJB for MOS Service xml processing.
 * <p>
 * 
 * @author Anamitra
 * 
 */

public class MOSServiceBean implements SessionBean
{
    private SessionContext ctx;

    public void ejbActivate()
    {
    }

    public void ejbCreate()
    {
    }

    public void ejbRemove()
    {
        // closeAllProducers()
    }

    public void ejbPassivate()
    {
    }

    public void setSessionContext(SessionContext ctx)
    {
        this.ctx = ctx;
    }

    public byte[] processMOS(byte[] reqMosData, String mosName)
    {
        try
        {
            return secureProcessMOSInternal(SecurityUtil.getUserInfo(), reqMosData,
                mosName);
        }
        catch(MXException mxe)
        {
            throw new EJBException(mxe);
        }
    }
    
    public byte[] wsProcessMOS(byte[] reqMosData, String wsName)
    {
        try
        {
            WSRegistryInfo info = WSRegistryCache.getInstance().getWSRegistryInfo(wsName);
            if(info == null)
            {
            	String[] params = {wsName};
            	throw new MXApplicationException("iface","unregisteredservice",params);
            }

            return this.processMOS(reqMosData, info.getServiceName());
        }
        catch(MXException mxe)
        {
            throw new EJBException(mxe);
        }
    }


    public byte[] secureProcessMOS(byte[] reqMosData, String mosName) 
    {
        try
        {
            UserInfo userInfo = SecurityUtil.getUserInfo(null, null, ctx.getCallerPrincipal());
            return secureProcessMOSInternal(userInfo, reqMosData, mosName);
        }
        catch(MXException mxe)
        {
            throw new EJBException(mxe);
        }
    }
    
    public byte[] wsSecureProcessMOS(byte[] reqMosData, String wsName) 
    {
        try
        {
            WSRegistryInfo info = WSRegistryCache.getInstance().getWSRegistryInfo(wsName);
            if(info == null)
            {
            	String[] params = {wsName};
            	throw new MXApplicationException("iface","unregisteredservice",params);
            }

            return this.secureProcessMOS(reqMosData, info.getServiceName());
        }
        catch(MXException mxe)
        {
            throw new EJBException(mxe);
        }
    }

    
    public byte[] maxSecureProcessMOS(String loginid, String password,
            byte[] reqMosData, String mosName) 
    {
    	UserInfo userInfo = null;
        try
        {
            userInfo = SecurityUtil.getUserInfo(loginid, password, (java.security.Principal)null);
            return secureProcessMOSInternal(userInfo, reqMosData, mosName);
        }
        catch(MXException mxe)
        {
            throw new EJBException(mxe);
        }
        finally
        {
        	try
        	{
            	SecurityUtil.disconnectUser(userInfo);
        	}
        	catch(Exception e)
        	{
        		MicUtil.INTEGRATIONLOGGER.error(e.getMessage(), e);
        	}

        }
    }


    public byte[] wsMaxSecureProcessMOS(String loginid, String password,
            byte[] reqMosData, String wsName) 
    {
        try
        {
            WSRegistryInfo info = WSRegistryCache.getInstance().getWSRegistryInfo(wsName);
            if(info == null)
            {
            	String[] params = {wsName};
            	throw new MXApplicationException("iface","unregisteredservice",params);
            }

            return this.maxSecureProcessMOS(loginid, password, reqMosData, info.getServiceName());
        }
        catch(MXException mxe)
        {
            throw new EJBException(mxe);
        }
    }
    
    protected String getBidiFormat(String mosName)
    {
    	try
    	{
    		String bidiFormat = null;
    		if(!((MaxPropCache)MXServer.getMXServer().getFromMaximoCache("MAXPROP")).
    				getProperties().containsKey("mxe.int.bidi.os."+mosName.toLowerCase()))
    		{
    			bidiFormat = MXServer.getMXServer().getProperty("mxe.int.bidi.os");
    		}
    		else
    		{
    			bidiFormat = MXServer.getMXServer().getProperty("mxe.int.bidi.os."+mosName.toLowerCase());
    		}
    		
    		return bidiFormat;
    	}
    	catch(RemoteException re)
    	{
    		MicUtil.INTEGRATIONLOGGER.error(re.getMessage(),re);
    		return null;
    	}
    }

    protected Document transformToMaximoBidiFormatFrom(Document doc, String bidiFormat)
    	throws MXException, UnsupportedEncodingException
    {
    	if(bidiFormat != null && bidiFormat.trim().length() >0 && !bidiFormat.equalsIgnoreCase(MaximoBidiTransformer.MAXIMO_INBOUND_ILYNN))
    	{
    		doc = MaximoBidiTransformer.transformToMaximoFormat(doc, bidiFormat);
    		if(MicUtil.INTEGRATIONLOGGER.isDebugEnabled())
    		{
    			MicUtil.INTEGRATIONLOGGER.debug(new String(XMLUtils.convertDocumentToBytes(doc),"UTF-8"));
    		}
    		return doc;
    	}
    	return doc;
    }

    protected byte[] transformFromMaximoBidiFormatTo(byte[] responseData, String bidiFormat)
		throws MXException, UnsupportedEncodingException
	{
		if(bidiFormat != null && bidiFormat.trim().length() >0 && !bidiFormat.equalsIgnoreCase(MaximoBidiTransformer.MAXIMO_OUTBOUND_ICYNN))
		{
			Document doc = XMLUtils.convertBytesToDocument(responseData);
			doc = MaximoBidiTransformer.transformFromMaximoFormat(doc, bidiFormat);
			if(MicUtil.INTEGRATIONLOGGER.isDebugEnabled())
			{
				MicUtil.INTEGRATIONLOGGER.debug(new String(XMLUtils.convertDocumentToBytes(doc),"UTF-8"));
			}
			return XMLUtils.convertDocumentToBytes(doc);
		}
		return responseData;
	}
    
    
    private void setThreadLogging(UserInfo userInfo)
    {
    	LogInfo logInfo = ThreadLoggerCache.getInstance().getLogInfo("OSMIF."+userInfo.getUserName());
    	if(logInfo != null)
    	{
    		MaxLoggerContext.createCurrentContext(logInfo, ""+psdi.iface.jms.MessageUtil.generateUniqueID(), userInfo);
    	}
    }



    private byte[] secureProcessMOSInternal(UserInfo userInfo,
            byte[] reqMosData, String mosName) throws MXException
    {
        if (reqMosData == null)
        {
            throw new MXApplicationException("iface", "nullreqxml");
        }
        try
        {
            if (MXServer.getMXServer().isAdminModeOn(false))
            {
    			throw new MXApplicationException("system", "NoLogin", new Object[] {userInfo.getLoginID()});
            }
        	this.setThreadLogging(userInfo);

//            QName qname = MXStAXParser.getInstance().getRootElementName(
//                reqMosData);
            Document reqMosdoc = XMLUtils.convertBytesToDocument(reqMosData);
            String name = reqMosdoc.getRootElement().getName();
            //String ns = doc.getRootElement().getNamespaceURI();
            //QName qname = new QName(ns,name);
        	String bidiFormat = this.getBidiFormat(mosName);
        	reqMosdoc = this.transformToMaximoBidiFormatFrom(reqMosdoc, bidiFormat);
            String messageType = getMessageType(name, mosName);
            byte[] respMosData = null;
            if (messageType.equals(MosConstants.MESSAGETYPE_QUERY))
            {
                respMosData = ((MicService)MXServer.getMXServer().lookup("MIC"))
                        .query(reqMosdoc, mosName, userInfo);
                respMosData = this.transformFromMaximoBidiFormatTo(respMosData, bidiFormat);
            }
            else
            {
                respMosData = ((MicService)MXServer.getMXServer().lookup("MIC"))
                	.processObjectStructureService(reqMosdoc, mosName,
                			messageType, userInfo, null);
                if(respMosData == null)
                {
                    respMosData = StAXUtil.createEmptyResponse(messageType + mosName + MosConstants.MESSAGETYPE_RESPONSE, userInfo);
                }
                else
                {
                	respMosData = this.transformFromMaximoBidiFormatTo(respMosData, bidiFormat);
                }
            }
            return respMosData;
        }
        catch (RemoteException re)
        {
            throw new MXApplicationException("system", "remoteexception", re);
        }
        catch (UnsupportedEncodingException e)
        {
        	throw new EJBException(e);
		}
        finally
        {
    		MaxLoggerContext.destroyCurrentContext();
        }

    }

    private String getMessageType(String rootElemName, String mosName)
    {
        //String rootElemName = qname.getLocalPart();
        int index = rootElemName.indexOf(mosName);
        String messageType = rootElemName.substring(0, index);
        return messageType;
    }

}
